package com.bcxin.rest.web.apis.controllers;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.comparator.VersionComparator;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bcxin.Infrastructures.components.RetryProvider;
import com.bcxin.Infrastructures.exceptions.BadTenantException;
import com.bcxin.Infrastructures.utils.BjNwJwtUtil;
import com.bcxin.Infrastructures.utils.JwtCommonUtil;
import com.bcxin.Infrastructures.utils.JwtUtil;
import com.bcxin.Infrastructures.utils.RedisUtil;
import com.bcxin.api.interfaces.RequestAbstract;
import com.bcxin.api.interfaces.commons.ConfigRpcProvider;
import com.bcxin.api.interfaces.enums.LoginFromType;
import com.bcxin.api.interfaces.identities.IdentityRpcProvider;
import com.bcxin.api.interfaces.identities.requests.*;
import com.bcxin.api.interfaces.identities.responses.PrepareResetPasswordResponse;
import com.bcxin.api.interfaces.identities.responses.SignInResponse;
import com.bcxin.api.interfaces.tenants.SingleLoginRpcProvider;
import com.bcxin.api.interfaces.tenants.responses.LoginUserResponse;
import com.bcxin.rest.web.apis.configs.AreaCodeConfig;
import com.bcxin.rest.web.apis.configs.BJSSOConfig;
import com.bcxin.rest.web.apis.configs.V5AppConfig;
import com.bcxin.rest.web.apis.utils.*;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.util.StopWatch;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

@RestController
@RequestMapping("/identity")
public class IdentityController extends ControllerAbstract {


    private final Logger logger = LoggerFactory.getLogger(IdentityController.class);

    private final IdentityRpcProvider identityRpcProvider;

    private final SingleLoginRpcProvider singleLoginRpcProvider;

    private final ConfigRpcProvider configRpcProvider;

    private final RetryProvider retryProvider;

    private final RedisUtil redisUtil;
    private final V5AppConfig v5AppConfig;
    private final AreaCodeConfig areaCodeConfig;

    @Value("${QRCodeAuth:defaultValue}")
    public  String QRCodeAuth;
    @Value("${randomFrom:defaultValue}")
    public  String randomFrom;

    public IdentityController(IdentityRpcProvider identityRpcProvider,
                              SingleLoginRpcProvider singleLoginRpcProvider,
                              ConfigRpcProvider configRpcProvider,
                              RetryProvider retryProvider,
                              RedisUtil redisUtil, V5AppConfig v5AppConfig,AreaCodeConfig areaCodeConfig) {
        this.identityRpcProvider = identityRpcProvider;
        this.singleLoginRpcProvider = singleLoginRpcProvider;
        this.configRpcProvider = configRpcProvider;
        this.retryProvider = retryProvider;
        this.redisUtil = redisUtil;
        this.v5AppConfig = v5AppConfig;
        this.areaCodeConfig=areaCodeConfig;
    }

    @ApiOperation(value = "登入", response = SignInResponse.class)
    @PostMapping("/sign-in")
    public ResponseEntity<SignInResponse> signIn(@Valid @RequestBody SignInRequest request, HttpServletRequest httpServletRequest) {
        StopWatch stopWatch = new StopWatch();
        //app登录需要先判断;
        try {
            stopWatch.start();
            if (request.isFromMobile()) {
                ResponseEntity responseEntity = this.signBeforeChick(request);

                if (responseEntity != null) {
                    return responseEntity;
                }
            }
            request.setIpAddress(IPUtils.getIpAddress(httpServletRequest));
            SignInResponse response = this.identityRpcProvider.signIn(request);

            return this.ok(response);
        } catch (Exception ex) {
            try {
                stopWatch.stop();
                logger.error("failed to sign in with username={}；total cost={} ms;", request.getUserName(), stopWatch.getTotalTimeMillis());
            }catch (Exception e){}


            throw ex;
        }
    }


    @ApiOperation(value = "融通单点登录")
    @GetMapping("/rt/sso/login")
    public String rtSsoLogin(@RequestParam("ticket") String ticket,@RequestParam(value = "path",required = false) String path, HttpServletRequest httpServletRequest, HttpServletResponse response) {


        //获取接口调用地址
        String rtThirdPartyUrl = configRpcProvider.getValueByKey("rtThirdPartyUrl");
        SignInResponse signInResponse = null;
        try {
            String url = rtThirdPartyUrl + "/seeyon/thirdpartyController.do?ticket=" + ticket;
            Map<String, List<String>> headers = HttpUtil.createGet(url).execute().headers();

            //获取用户名
            List<String> loginNameList = headers.get("LoginName");

            if (loginNameList != null && loginNameList.size() > 0) {
                String loginName = loginNameList.get(0);

                SsoLoginRequest request = new SsoLoginRequest();
                request.setUserName(loginName);
                request.setIpAddress(IPUtils.getIpAddress(httpServletRequest));
                signInResponse = identityRpcProvider.ssoLogin(request);
                //转换url
                String urlPath = "";
                if (StrUtil.isNotEmpty(path)) {
                     urlPath = URLEncoder.encode(path, "UTF-8");
                }
                response.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?homeTemplateBoradType=rtpayTemplate&__network__=false&token=" + signInResponse.getToken()+"&path="+urlPath);
            } else {
                response.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/index.html");
            }

        } catch (Exception e) {
            try {
                response.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/index.html");
            } catch (Exception e1) {
                return "SSOLogoutError";
            }
        }

        return "SSOOK";

    }
    /**
     * 湖南单点
     */
    @GetMapping("/hunan/sso/login")
    public String hunanssoLogin(@RequestParam("userToken") String userToken, HttpServletRequest httpServletRequest,HttpServletResponse response) throws UnsupportedEncodingException {
        try {
            httpServletRequest.setCharacterEncoding("utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        response.setCharacterEncoding("utf-8");
        SignInResponse signInResponse = null;
        if(!JwtCommonUtil.validationToken(userToken)){
            return "SSOLogoutError:validFail";
        }
        Map<String,String> hashMap = JwtCommonUtil.parseToken(userToken);
        String idNum = hashMap.get("sfz");
        SsoLoginRequest request = new SsoLoginRequest();
        request.setUserName(idNum);
        request.setIpAddress(IPUtils.getIpAddress(httpServletRequest));
        try {
            signInResponse = identityRpcProvider.ssoLogin(request);
        } catch (Exception e) {
            return e.getMessage();
        }
        try {
            response.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?token=" + signInResponse.getToken());
        } catch (IOException e) {
            return "SSOLogoutError";
        }
        return "SSOOK";
    }

    @GetMapping("/hunan/out/sso/login")
    public String hunanOutSsoLogin(@RequestParam("userToken") String userToken, @RequestParam("state") String state, HttpServletRequest httpServletRequest,HttpServletResponse response) throws UnsupportedEncodingException {
        try {
            httpServletRequest.setCharacterEncoding("utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        response.setCharacterEncoding("utf-8");
        SignInResponse signInResponse;
        userToken = Base64.decodeStr(userToken);
        if(StringUtils.isEmpty(userToken)){
            throw new BadTenantException("userToken参数内容不能为空！");
        }
        if(StringUtils.isEmpty(state)){
            throw new BadTenantException("state参数内容不能为空！");
        }
        Map<String,String> hashMap = JSON.parseObject(userToken,Map.class);
        String idNum = JwtCommonUtil.decryptSm4Token(hashMap.get("idNum"));
        if(StringUtils.isEmpty(idNum)){
            throw new BadTenantException("idNum参数内容不能为空！");
        }
        String name = JwtCommonUtil.decryptSm4Token(hashMap.get("name"));
        String phone = JwtCommonUtil.decryptSm4Token(hashMap.get("phone"));
        logger.error(">>>>>>> 获取当前入参，state："+state+"，userToken："+userToken+"，idNum："+idNum+"，name："+name+"，phone："+phone);

        //获取用户，没有就自动创建
        String userName = singleLoginRpcProvider.saveLoginUserForLegalPerson(name,idNum,phone);

        SsoLoginRequest request = new SsoLoginRequest();
        request.setUserName(userName);
        request.setIpAddress(IPUtils.getIpAddress(httpServletRequest));
        try {
            signInResponse = identityRpcProvider.ssoLogin(request);
        } catch (Exception e) {
            return e.getMessage();
        }
        try {
            String type = "&type="+state;
            response.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?token=" + signInResponse.getToken()+type);
//            response.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?token=" + signInResponse.getToken());
        } catch (IOException e) {
            return "SSOLogoutError";
        }
        return "SSOOK";
    }

    /**
     * 广西、南宁单点
     *
     * @param request
     * @return
     */
    @ApiOperation(value = "广西、南宁单点登录", response = SignInResponse.class)
    @PostMapping("/sso/login")
    public ResponseEntity<SignInResponse> ssoLogin(@Valid SsoLoginRequest request, HttpServletRequest httpServletRequest) {
        SignInResponse response = null;
        try {
            request.setUserNameForSSO();
            singleLoginRpcProvider.saveLoginUser(request.getUserName());
            request.setIpAddress(IPUtils.getIpAddress(httpServletRequest));
            response = this.identityRpcProvider.ssoLogin(request);
        } catch (Exception e) {
            return this.error(e.getMessage());
        }
        return this.ok(response);
    }

    @ApiOperation(value = "北京CA认证登录", response = SignInResponse.class)
    @RequestMapping("/ca/login")
    public void caLogin(@Valid CALoginRequest loginRequest, HttpServletRequest request, HttpServletResponse response) {
        try {
            singleLoginRpcProvider.checkAuth();
            logger.error("获取当前地区编码："+areaCodeConfig.getAreaCode());
            if (areaCodeConfig.getAreaCode().equals("110100")) {
                String dnname = request.getHeader("dnname");
                if (StrUtil.isNotEmpty(dnname)) {
                    loginRequest.setUserNameForBJCA(dnname);
                }
            } else if (areaCodeConfig.getAreaCode().equals("430000")) {
                Cookie[] cookies = request.getCookies();
                logger.error("湖南数字证书获取cookie值："+cookies.toString());
                try {
                    if (cookies == null)
                        cookies = new Cookie[0];
                    for (int i = 0; i < cookies.length; i++) {
                        Cookie cookie = cookies[i];
                        if ("KOAL_CERT_CN".equals(cookie.getName())) {
                            logger.error("cookie获取用户名信息："+URLDecoder.decode(cookie.getValue(),"UTF-8"));
                            String userName= URLDecoder.decode(cookie.getValue(),"UTF-8");
                            loginRequest.setUserNameForHNCA(userName);
                        }
                    }
                } catch (UnsupportedEncodingException e) {
                    throw new BadTenantException(String.format("湖南数字证书异常;Exception=%s", e.toString()));
                }

            }
            logger.error("湖南数字证书获取用户名信息："+loginRequest.getUserName());
            if (StrUtil.isNotEmpty(loginRequest.getUserName())) {
                if (areaCodeConfig.getAreaCode().equals("110100")) {
                    LoginUserResponse loginUserResponse = singleLoginRpcProvider.findByIdNum(loginRequest.getUserName());
                    loginRequest.setUserName(loginUserResponse.getTelephone());
                }
                SignInResponse signInResponse = this.identityRpcProvider.ssoLoginByIdNum(loginRequest.getUserName(), IPUtils.getIpAddress(request));
                logger.error("登录成功跳转地址："+configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?token=" + signInResponse.getToken());
                response.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?token=" + signInResponse.getToken());
            } else {
                response.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/");
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
            try {
                response.setCharacterEncoding("GB2312");
                response.getWriter().write(e.getMessage());
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
    }

    @ApiOperation(value = "北京内外单点登录", response = SignInResponse.class)
    @GetMapping("/beijing/in/sso/login")
    public String beijingInSsoLogin(@RequestParam("token") String token, HttpServletRequest httpServletRequest,HttpServletResponse response) throws UnsupportedEncodingException {
        try {
            httpServletRequest.setCharacterEncoding("utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        logger.info(">>>>>>> 获取当前入参，userToken：{}",token);
        response.setCharacterEncoding("utf-8");
        SignInResponse signInResponse;
        Map<String,String> hashMap = new BjNwJwtUtil().verifyToken(token);
        String idNum = hashMap.get("idNo");
        if(StringUtils.isEmpty(idNum)){
            throw new BadTenantException("idNum参数内容不能为空！");
        }
        String name = hashMap.get("name");
        String organCode = hashMap.get("organCode");
        String organName = hashMap.get("organName");
        String orglevel = hashMap.get("orglevel");
        String ukeySn = hashMap.get("ukeySn");
        logger.info(">>>>>>> 获取当前入参，userToken：{}，hashMap：{}",token,hashMap);

        //获取用户，没有就自动创建
//        String userName = singleLoginRpcProvider.saveLoginUserForLegalPerson(name,idNum,"");
        String userName = idNum;
        SsoLoginRequest request = new SsoLoginRequest();
        request.setUserName(userName);
        request.setIpAddress(IPUtils.getIpAddress(httpServletRequest));
        try {
            signInResponse = identityRpcProvider.ssoLogin(request);
        } catch (Exception e) {
            return e.getMessage();
        }
        try {
            String type = "";
//            String type = "&type=";
            response.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?token=" + signInResponse.getToken()+type);
        } catch (IOException e) {
            return "SSOLogoutError";
        }
        return "SSOOK";
    }

    @ApiOperation(value = "北京自然人单点登录", response = SignInResponse.class)
    @GetMapping("/sso/login/naturalPerson")
    public void ssoLoginNaturalPerson(HttpServletRequest req, HttpServletResponse res, SsoLoginRequest request) throws IOException {
        res.setContentType("text/html;charset=utf-8");
        res.setCharacterEncoding("UTF-8");
        PrintWriter pw = res.getWriter();
        SignInResponse response = null;
        try {
            Object userInfo = redisUtil.get("sso_natural:" + req.getParameter("code"));
            String userName = null;
            if (userInfo != null) {
                userName = userInfo.toString();
            } else {
                userName = singleLoginRpcProvider.saveLoginUserForNaturalPerson(request.getCode());
            }
            request.setUserName(userName);
            request.setIpAddress(IPUtils.getIpAddress(req));
            response = this.identityRpcProvider.ssoLogin(request);
            String state = req.getParameter("state");
            /*switch (state) {
                case "v501":
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/portal/vue/index.html#/open?appId=__KDXZvc8mnEmDqMARhK7G&actionContent=__vJplXNdVxgo5Rpyif75&opentarget=detail&linkType=00&flag=1&token=" + response.getToken());
                    break;
                case "v502":
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/portal/vue/index.html#/open?appId=__KDXZvc8mnEmDqMARhK7G&actionContent=__AVYis7QN4iLJYRXNGhE&opentarget=detail&linkType=00&flag=1&token=" + response.getToken());
                    break;
                case "v503":
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/portal/vue/index.html#/open?appId=__KDXZvc8mnEmDqMARhK7G&actionContent=__zagjldcIQyPSATrE9me&opentarget=detail&linkType=00&flag=1&token=" + response.getToken());
                    break;
                case "v504":
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/portal/vue/index.html#/open?appId=__KDXZvc8mnEmDqMARhK7G&actionContent=__ApIdjDPXF5g3dIg9LMM&opentarget=detail&linkType=00&flag=1&token=" + response.getToken());
                    break;
                case "v505":
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/portal/vue/index.html#/open?appId=__KDXZvc8mnEmDqMARhK7G&actionContent=__6sVq2e7D6SVInCgisRS&opentarget=detail&linkType=00&flag=1&token=" + response.getToken());
                    break;
                case "v506":
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/portal/vue/index.html#/open?appId=__KDXZvc8mnEmDqMARhK7G&actionContent=__PX0OFiN9BKKt6hjbEy9&opentarget=detail&linkType=00&flag=1&token=" + response.getToken());
                    break;
                default:
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?token=" + response.getToken());
                    break;
            }*/
            String type = "&type="+state;
            res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?token=" + response.getToken()+type);
        } catch (Exception e) {
            pw.write("<br/>");
            pw.write("<h3>" + e.getMessage() + "</h3>");
        }
    }

    @ApiOperation(value = "获取北京自然人单点登录跳转地址")
    @GetMapping("/sso/redirectUrl/naturalPerson")
    public ResponseEntity getRedirectUrlForNaturalPerson() throws UnsupportedEncodingException {
        return this.ok(singleLoginRpcProvider.getRedirectUrlForNaturalPerson());
    }

    @ApiOperation(value = "获取北京法人单点登录跳转地址")
    @GetMapping("/sso/redirectUrl/legalPerson")
    public ResponseEntity getRedirectUrlForLegalPerson() {
        return this.ok(BJSSOConfig.redirectUrl);
    }

    @ApiOperation(value = "北京法人单点登录", response = SignInResponse.class)
    @GetMapping("/sso/login/legalPerson")
    public void ssoLoginLegalPerson(HttpServletRequest req, HttpServletResponse res, SsoLoginRequest request) throws IOException {
        res.setContentType("text/html;charset=utf-8");
        res.setCharacterEncoding("UTF-8");
        PrintWriter pw = res.getWriter();
        SignInResponse response = null;
        try {
            Object userInfo = redisUtil.get("sso:" + req.getParameter("code"));
            String userInfoStr = null;
            if (userInfo == null) {
                userInfoStr = new YZTAuthUtil().doAuth(req, true);
                if (StrUtil.isEmpty(userInfoStr)) {
                    pw.print("获取用户的信息为空");
                    pw.print(req.getAttribute("yzt_login_ermsg"));
                } else {
                    redisUtil.set("sso:" + req.getParameter("code"), userInfoStr, 1800);
                }
            } else {
                userInfoStr = userInfo.toString();
            }

            logger.error(userInfoStr);
            JSONObject userInfoJson = JSONObject.parseObject(userInfoStr);

            String errorMsg = userInfoJson.getString("error_msg");
            if (StrUtil.isNotEmpty(errorMsg)) {
                pw.write("<br/>");
                pw.write("<h3>" + errorMsg + "</h3>");
            }

            String extProperties = userInfoJson.getString("extProperties");
            String idNum = YZTAuthUtil.resolveExtProperties(extProperties, "LEGAL_PERSON_ID");   // 法人证件号
            request.setUserName(idNum);
            request.setIpAddress(IPUtils.getIpAddress(req));
            response = this.identityRpcProvider.ssoLogin(request);
            res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?token=" + response.getToken());
        } catch (Exception e) {
            pw.write("<br/>");
            pw.write("<h3>" + e.getMessage() + "</h3>");
        }
    }

    @ApiOperation(value = "北京法人单点登录（百保盾过来）", response = SignInResponse.class)
    @GetMapping("/sso/bbd/ssoLegalLogin")
    public String ssoBBDLogin(String token, HttpServletRequest request) {
        String paramStr = JwtCommonUtil.parseJWT1(token);
        if(StringUtils.isEmpty(paramStr)){
            return "";
        }
        Map<String,String> paramMap = JSON.parseObject(paramStr, Map.class);
        String username = singleLoginRpcProvider.saveLoginUserForLegalPerson(paramMap.get("certName"),paramMap.get("certNo"),paramMap.get("mobile"));
        AtomicReference<String> result = new AtomicReference<>();
        this.retryProvider.execute(() -> {
            SignInResponse response = null;
            try {
                response = this.identityRpcProvider.ssoLoginByIdNum(username, IPUtils.getIpAddress(request));
            } catch (Exception e) {
                ThreadUtil.sleep(1000);
                logger.error("北京法人单点登录（百保盾过来）业务失败，ERR:"+e.getMessage());
                throw new BadTenantException(e.getMessage());
            }
            if(response!=null&&StringUtils.isNotEmpty(response.getToken())){
                result.set(response.getToken());
            }else{
                ThreadUtil.sleep(1000);
                throw new BadTenantException("法人信息未入库，轮询中...");
            }
        }, 3);
        return result.get();
    }

    @ApiOperation(value = "陕西政务网单点登录", response = SignInResponse.class)
    @GetMapping("/shanxi/out/sso/login")
    public void shanxiOutSsoLogin(HttpServletRequest req, HttpServletResponse res) throws IOException {
        res.setContentType("text/html;charset=utf-8");
        res.setCharacterEncoding("UTF-8");
        PrintWriter pw = res.getWriter();
        SignInResponse response = null;
        try {
            String authToken = req.getParameter("authToken");
            String state = req.getParameter("state");
            if(StringUtils.isEmpty(authToken)){
                throw new BadTenantException("authToken参数内容不能为空！");
            }
            Map<String,String> paramMap = JwtCommonUtil.SXZW_verifyToken(authToken);
            String userName = null;
            if(StringUtils.isEmpty(paramMap.get("PERSONID"))){
                throw new BadTenantException("PERSONID参数内容不能为空！");
            }
            if("0".equals(paramMap.get("TYPE"))){
                Object userInfo = redisUtil.get("sso_natural:" + paramMap.get("PERSONID"));
                if (userInfo != null) {
                    userName = userInfo.toString();
                } else {
                    userName = singleLoginRpcProvider.saveLoginUserForNaturalPerson(paramMap.get("PERSON"),paramMap.get("PERSONID"),paramMap.get("MOBILE"));
                }
            }else{
                userName = singleLoginRpcProvider.saveLoginUserForLegalPerson(paramMap.get("LEGALPERSON"),paramMap.get("LEGALPERSONID"),paramMap.get("BLRMOBILE"));
            }
            ThreadUtil.sleep(3000);
            SsoLoginRequest request = new SsoLoginRequest();
            request.setUserName(userName);
            request.setIpAddress(IPUtils.getIpAddress(req));
            response = this.identityRpcProvider.ssoLogin(request);
            /*switch (state) {
                case "v501":
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/portal/vue/index.html#/open?appId=__KDXZvc8mnEmDqMARhK7G&actionContent=__vJplXNdVxgo5Rpyif75&opentarget=detail&linkType=00&flag=1&token=" + response.getToken());
                    break;
                case "v502":
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/portal/vue/index.html#/open?appId=__KDXZvc8mnEmDqMARhK7G&actionContent=__AVYis7QN4iLJYRXNGhE&opentarget=detail&linkType=00&flag=1&token=" + response.getToken());
                    break;
                case "v503":
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/portal/vue/index.html#/open?appId=__KDXZvc8mnEmDqMARhK7G&actionContent=__zagjldcIQyPSATrE9me&opentarget=detail&linkType=00&flag=1&token=" + response.getToken());
                    break;
                case "v504":
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/portal/vue/index.html#/open?appId=__KDXZvc8mnEmDqMARhK7G&actionContent=__ApIdjDPXF5g3dIg9LMM&opentarget=detail&linkType=00&flag=1&token=" + response.getToken());
                    break;
                case "v505":
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/portal/vue/index.html#/open?appId=__KDXZvc8mnEmDqMARhK7G&actionContent=__6sVq2e7D6SVInCgisRS&opentarget=detail&linkType=00&flag=1&token=" + response.getToken());
                    break;
                case "v506":
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/portal/vue/index.html#/open?appId=__KDXZvc8mnEmDqMARhK7G&actionContent=__PX0OFiN9BKKt6hjbEy9&opentarget=detail&linkType=00&flag=1&token=" + response.getToken());
                    break;
                default:
                    res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?token=" + response.getToken());
                    break;
            }*/
            //0212 自行招用保安员单位的备案;0213 保安服务公司设立分公司备案;0214 保安服务公司跨区域经营保安服务备案;0215 保安员证申领;0253 设立保安培训机构许可;0254 设立保安服务公司许可
            state = "021201".equals(state)?"v504":"021301".equals(state)?"v610":"021401".equals(state)?"v611":"021501".equals(state)?"v501":"025301".equals(state)?"v503":"025401".equals(state)?"v502":"";
            String type = "&type="+state;
            res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?token=" + response.getToken()+type);
        } catch (Exception e) {
            pw.write("<br/>");
            pw.write("<h3>" + e.getMessage() + "</h3>");
        }
    }


    @ApiOperation(value = "修改密码", response = SignInResponse.class)
    @PostMapping("/change-password")
    public ResponseEntity changePassword(
            @Valid @RequestBody com.bcxin.rest.web.apis.requests.ChangePasswordRequest request,
            HttpServletRequest servletRequest) {
        ChangePasswordRequest changePasswordRequest = request.getRequest(servletRequest);
        this.identityRpcProvider.changePassword(changePasswordRequest);

        return this.ok(null);
    }

    @ApiOperation(value = "微信登入", response = SignInResponse.class)
    @PostMapping("/wechat-in")
    public ResponseEntity<SignInResponse> sso_login(@RequestBody WeChatSignInRequest request, HttpServletRequest httpServletRequest) {

        ResponseEntity responseEntity = this.signBeforeChick(request);

        if (responseEntity!=null) {
            return responseEntity;
        }
        request.setIpAddress(IPUtils.getIpAddress(httpServletRequest));
        SignInResponse response = this.identityRpcProvider.signIn(request);

        return this.ok(response);
    }

    @ApiOperation(value = "微信小程序登入", response = SignInResponse.class)
    @PostMapping("/wechat-mini-in")
    public ResponseEntity<SignInResponse> wechat_mini_login(@RequestBody WeChatMiniSignInRequest request, HttpServletRequest httpServletRequest) {
        request.setIpAddress(IPUtils.getIpAddress(httpServletRequest));
        SignInResponse response = this.identityRpcProvider.signIn(request);
        return this.ok(response);
    }



    @ApiOperation(value = "忘记密码前的验证证件号是否存在")
    @PostMapping("/prepare-reset-password-exist")
    public ResponseEntity prepareResetPasswordExist(@RequestBody PrepareResetPasswordRequest request) {
        PrepareResetPasswordResponse response = this.identityRpcProvider.prepareResetPasswordExist(request);
        return this.ok(response);
    }

    @ApiOperation(value = "忘记密码前的验证操作(短信/人脸)")
    @PostMapping("/prepare-reset-password")
    public ResponseEntity prepareResetPassword(@RequestBody PrepareResetPasswordRequest request) {
        PrepareResetPasswordResponse response = this.identityRpcProvider.prepareResetPassword(request);
        return this.ok(response);
    }

    @ApiOperation(value = "通过忘记密码前的操作验证验证码")
    @PostMapping("/prepare-reset-password-check-code")
    public ResponseEntity prepareResetPasswordCheckCode(@RequestBody ResetPasswordRequest request) {
        this.identityRpcProvider.prepareResetPasswordCheckCode(request);

        return this.ok();
    }
    @ApiOperation(value = "通过忘记密码前的操作执行密码修改")
    @PostMapping("/reset-password")
    public ResponseEntity resetPassword(@RequestBody ResetPasswordRequest request) {
        this.identityRpcProvider.restPassword(request);

        return this.ok();
    }

    @ApiOperation(value = "通过密码和token解锁账号")
    @PostMapping("/unlock")
    public ResponseEntity unlock(@RequestBody UnlockRequest request) {
        String userId = this.getCurrentUserId();
        request.setUserid(userId);
        String token = this.identityRpcProvider.unlock(request);
        return this.ok(token);
    }
    @ApiOperation(value = "修改用户的登录名称")
    @PostMapping("/reset-username")
    public ResponseEntity resetUserName(@RequestBody ResetUserNameRequest request) {
        this.identityRpcProvider.restUsername(request);
        return this.ok();
    }
    //登录前校验
    public ResponseEntity  signBeforeChick(RequestAbstract request){
        //百保盾app登录进入这个分支
        if(request.getLoginFrom() == null || request.getLoginFrom() == LoginFromType.BBD){
            String v5Version = configRpcProvider.getValueByKeyNoCache("v5Version");
            String v5WgtVersion = configRpcProvider.getValueByKeyNoCache("v5WgtVersion");
            String v5UpdateMsg = configRpcProvider.getValueByKeyNoCache("v5UpdateMsg");
            if (StrUtil.isEmpty(request.getVersion()) || StrUtil.isEmpty(request.getWgtVersion())) {
                return this.status(HttpStatus.OK, v5UpdateMsg);
            }
            int compareVersion = VersionComparator.INSTANCE.compare(request.getVersion(), v5Version);
            int compareWgtVersion = VersionComparator.INSTANCE.compare(request.getWgtVersion(), v5WgtVersion);
            //判断大版本
            if (compareVersion < 0) {
                //return this.status(HttpStatus.OK, v5UpdateMsg);
            }
            //大版本一样判断小版本
            if (compareVersion == 0) {
                if (compareWgtVersion < 0) {
                    //return this.status(HttpStatus.OK, v5UpdateMsg);
                }
            }
        }

        //百课堂进入这个分支
        if (request.getLoginFrom() != null && request.getLoginFrom() == LoginFromType.BKT){
            String v5BktVersion = configRpcProvider.getValueByKeyNoCache("v5BktVersion");
            String v5BktWgtVersion = configRpcProvider.getValueByKeyNoCache("v5BktWgtVersion");
            String v5BktUpdateMsg = configRpcProvider.getValueByKeyNoCache("v5BktUpdateMsg");
            if (StrUtil.isEmpty(request.getVersion()) || StrUtil.isEmpty(request.getWgtVersion())) {
                return this.status(HttpStatus.OK, v5BktUpdateMsg);
            }
            int compareVersion = VersionComparator.INSTANCE.compare(request.getVersion(), v5BktVersion);
            int compareWgtVersion = VersionComparator.INSTANCE.compare(request.getWgtVersion(), v5BktWgtVersion);
            //判断大版本
            if (compareVersion < 0) {
                //return this.status(HttpStatus.OK, v5BktUpdateMsg);
            }
            //大版本一样判断小版本
            if (compareVersion == 0) {
                if (compareWgtVersion < 0) {
                    //return this.status(HttpStatus.OK, v5BktUpdateMsg);
                }
            }

        }

        return null;
    }
    @PostMapping("/shanxi/pki/login")
    @ResponseBody
    public AjaxResult shanXiPkiLogin(String username) {
        if (!username.equals("bbd_manager")){
            JitGatewayUtil jitGatewayUtil = new JitGatewayUtil();
            Object qrCodeAuthObj = QRCodeAuth;
            String qrCodeAuthStr = String.valueOf(qrCodeAuthObj);
            String randNum = null;
            if ("false".equals(qrCodeAuthStr)) {
                logger.info("生成原文开始");
                randNum = jitGatewayUtil.generateRandomNum(randomFrom);
                if (!jitGatewayUtil.isNotNull(randNum)) {
                    logger.info("生成原文为空！-----------获取原文失败");
                    return new AjaxResult(false, "获取原文失败");
                }
                logger.info("生成原文结束，成功生成原文：" + randNum);
            }
            Map data = new HashMap();
            data.put("original_data", randNum);
            data.put("QRCodeAuth", qrCodeAuthStr);
            data.put("original", randNum);
            return new AjaxResult(true, "成功", data);

        }else {
            return new AjaxResult(true, "成功", "login");
        }
    }
    @PostMapping("/gansu/pki/login")
    @ResponseBody
    public AjaxResult ganSuPkiLogin(String username) {
        if (!username.equals("gs_super")){
            JitGatewayUtil jitGatewayUtil = new JitGatewayUtil();
            Object qrCodeAuthObj = QRCodeAuth;
            String qrCodeAuthStr = String.valueOf(qrCodeAuthObj);
            String randNum = null;
            if ("false".equals(qrCodeAuthStr)) {
                logger.info("生成原文开始");
                randNum = jitGatewayUtil.generateRandomNum(randomFrom);
                if (!jitGatewayUtil.isNotNull(randNum)) {
                    logger.info("生成原文为空！-----------获取原文失败");
                    return new AjaxResult(false, "获取原文失败");
                }
                logger.info("生成原文结束，成功生成原文：" + randNum);
            }
            Map data = new HashMap();
            data.put("original_data", randNum);
            data.put("QRCodeAuth", qrCodeAuthStr);
            data.put("original", randNum);
            return new AjaxResult(true, "成功", data);

        }else {
            return new AjaxResult(true, "成功", "login");
        }
    }
    @RequestMapping("/jitGWAuth")
    @ResponseBody
    public AjaxResult authen(String authMode, String original, String signed_data, HttpServletRequest request) throws IOException {
        System.out.println("身份认证开始！\n"+"authMode:" + authMode +"original:"+original+"signed_data"+signed_data);
        JitGatewayUtil jitGatewayUtil = new JitGatewayUtil();
        jitGatewayUtil.jitGatewayUtilBean.setAuthMode("cert");
        jitGatewayUtil.jitGatewayUtilBean.setToken("");
        jitGatewayUtil.jitGatewayUtilBean.setOriginal_data(original);
        jitGatewayUtil.jitGatewayUtilBean.setOriginal_jsp(original);
        jitGatewayUtil.jitGatewayUtilBean.setSigned_data(signed_data);
        jitGatewayUtil.jitGatewayUtilBean.setRemoteAddr("");
        System.out.println("——————————————————开始调用auth——————————————");
        jitGatewayUtil.auth();
        System.out.println("——————————————————调用结束auth——————————————");
        if (!jitGatewayUtil.authResult.isSuccess()) {
            System.out.println("身份认证失败，失败原因：" + jitGatewayUtil.authResult.getErrDesc());
            return new AjaxResult(false, "身份认证失败，失败原因：" + jitGatewayUtil.authResult.getErrDesc());
        } else {

            System.out.println("身份认证成功，认证信息正常返回！\n");
            return new AjaxResult(true, "身份认证成功。");
        }
    }

    @ApiOperation(value = "甘肃政务网单点登录")
    @GetMapping("/gs/out/sso/login")
    public void   gsLoginLegalPerson(@RequestParam("ticket") String ticket, @RequestParam("state") String state,@RequestParam("loginType") String loginType,HttpServletRequest req, HttpServletResponse res) throws IOException {
        res.setContentType("text/html;charset=utf-8");
        res.setCharacterEncoding("UTF-8");
        SignInResponse response = null;
        PrintWriter pw = res.getWriter();
        if(StringUtils.isEmpty(ticket)){
            throw new BadTenantException("ticket参数内容不能为空！");
        }
        if(StringUtils.isEmpty(state)){
            throw new BadTenantException("state参数内容不能为空！");
        }
        if(StringUtils.isEmpty(loginType)){
            throw new BadTenantException("loginType参数内容不能为空！");
        }
        SsoLoginRequest request = new SsoLoginRequest();
        try {
            String userinfoUrl ="https://zwfw.gat.gansu.gov.cn/seeyon/thirdpartyController.do?ticket=" + ticket;
            String tokenUrl="https://zwfw.gat.gansu.gov.cn/zw_portal_api/third/expose/getAccessToken?serviceCode=BAXKBA";
            HttpRequest reqUserinfo= HttpUtil.createGet(userinfoUrl);
            HttpRequest reqToken= HttpUtil.createGet(tokenUrl);
            // 发起请求
            HttpResponse resToken= reqToken.execute();
            logger.info("获取token响应："+resToken);
            JSONObject jsonToken=JSON.parseObject(resToken.body());
            if(jsonToken.getString("code")!="200"){
                throw new IOException("获取token失败："+jsonToken.getString("message"));
            }
            // 添加请求头
            reqUserinfo.header("appToken", jsonToken.getString("result"));
            HttpResponse resUserinfo = reqUserinfo.execute();
            logger.info("获取用户信息响应："+resUserinfo);
            if(jsonToken.getString("code")!="200"){
                throw new IOException("获取用户信息失败："+jsonToken.getString("message"));
            }
            JSONObject jsonUserinfo=JSON.parseObject(resUserinfo.body());
            JSONObject result=JSON.parseObject(jsonUserinfo.getString("result"));
            Object userInfo = redisUtil.get("sso_natural:" + result.getString("sfzh"));
            String userName = null;
            if("0".equals(loginType)) {
                if (userInfo != null) {
                    userName = result.getString("sfzh");
                } else {
                    userName = singleLoginRpcProvider.saveLoginUserForNaturalPerson(result.getString("xm"),result.getString("sfzh"),result.getString("sjhm"));
                }
            }else {
                userName = singleLoginRpcProvider.saveLoginUserForLegalPerson(result.getString("xm"),result.getString("sfzh"),result.getString("sjhm"));
            }
            ThreadUtil.sleep(3000);
            request.setUserName(userName);
            request.setIpAddress(IPUtils.getIpAddress(req));
            response = this.identityRpcProvider.ssoLogin(request);
            //0212 自行招用保安员单位的备案;0213 保安服务公司设立分公司备案;0214 保安服务公司跨区域经营保安服务备案;0215 保安员证申领;0253 设立保安培训机构许可;0254 设立保安服务公司许可
            state = "021201".equals(state)?"v504":"021301".equals(state)?"":"021401".equals(state)?"v505":"021501".equals(state)?"v501":"025301".equals(state)?"v503":"025401".equals(state)?"v502":"";
            String type = "&type="+state;
            res.sendRedirect(configRpcProvider.getValueByKey("v5WebUrl") + "/static/signon/bjca.html?token=" + response.getToken()+type);
        } catch (Exception e) {
            pw.write("<br/>");
            pw.write("<h3>" + e.getMessage() + "</h3>");
        }
    }

}
